% function [f,J,] = dXdT(t,x,T,BX,K_BX,par)
%
%
%
% Output parameters:
%   f     time derivatives of the model
%   J     flux
%
%
% Mandatory input parameters:
%   t     time
%   x     state variables at t=0
%   T     temperature in degreesCelcius
%   BX    buffer sizes
%   K_BX  proton buffer dissociation constants ( Matrix )
%   par   parameter vector for the free parameters
%   
%
% State Variables:
% [citrate_Matrix , isocitrate_Matrix ]
%
%
% Free Parameters:
% [Vmf_ACON ]

function [f,J,varargout] = dXdT(t,x,T,BX,K_BX,par)
%% GLOBAL VARIABLES
%% LIST OF STATE VARIABLES
% 1 citrate_Matrix
% 2 isocitrate_Matrix
% 3 h_Matrix
% 4 m_Matrix
% 5 k_Matrix

% PARTIAL VOLUME FRACTIONS
VWater_Matrix = 1; % [=] l water (l region)^{-1}, Vinnakota and Bassingthwaighte, AJP, 2004 
VRegion_Matrix = 1; % [=] l region (l tissue)^{-1}, Vinnakota and Bassingthwaighte, AJP, 2004 

%% THERMODYNAMIC DATA
RT = 8.314*(T+273.15)/1e3; % kJ  mol^{-1}
F = 0.096484; % kJ mol^{-1} mV^{-1}

%% STATE VARIABLES

% Concentrations of Reference Species
citrate_Matrix = x(1);
isocitrate_Matrix = x(2);
% Concentrations of H, Mg, and K
h_Matrix = x(3);
m_Matrix = x(4);
k_Matrix = x(5);

% Membrane potentials

%% DISSOCIATION CONSTANTS
% citrate_Matrix
Kh(1) = 2.8039071084602661728529815476917974592652171850204e-06;
Km(1) = 0.00053152934130882352904007293048493920650798827409744;
Kk(1) = 0.32943036060075353566034550567565020173788070678711;
% isocitrate_Matrix
Kh(2) = 2.2530126058297659183968838364497955240040028002113e-06;
Km(2) = 0.0040787514698624521489178640365480532636865973472595;
Kk(2) = Inf;

%% BINDING POLYNOMIALS
P( 1 ) = 1  + h_Matrix/Kh(1) + m_Matrix/Km(1) + k_Matrix/Kk(1);
P( 2 ) = 1  + h_Matrix/Kh(2) + m_Matrix/Km(2) + k_Matrix/Kk(2);

%% THERMODYNAMIC EQUATIONS
DGro_ACON =6.65;

Keq_ACON_Matrix = exp(-DGro_ACON/RT)/P(1)*P(2);

%% FLUX EQUATIONS
%ACON_Matrix
a=citrate_Matrix;
p=isocitrate_Matrix;
KmA=1161e-6;
KmP=434e-6;
Vmf_ACON=par(1);
Vmr=Vmf_ACON*(KmP/KmA/Keq_ACON_Matrix);
J_f=Vmf_ACON*Vmr*a/(KmA*Vmr+Vmr*a+Vmf_ACON/Keq_ACON_Matrix*p);
J_ACON_Matrix=J_f-Vmf_ACON*Vmr*(p/Keq_ACON_Matrix)/(KmA*Vmr+Vmr*a+Vmf_ACON/Keq_ACON_Matrix*p);

%% REACTANT TIME DERIVATIVES
f(1,:) = ( 0  - 1*J_ACON_Matrix ) / VWater_Matrix; % citrate_Matrix
f(2,:) = ( 0  + 1*J_ACON_Matrix ) / VWater_Matrix; % isocitrate_Matrix

%% ION EQUATIONS
% COMPARTMENT Matrix:
ii = [1  2]; % Indices of SVs in compartment Matrix
% PARTIAL DERIVATIVES
pHBpM = -sum( (h_Matrix*x(ii)'./Kh(ii))./(Km(ii).*P(ii).^2) );
pHBpK = -sum( (h_Matrix*x(ii)'./Kh(ii))./(Kk(ii).*P(ii).^2) );
pHBpH = +sum( (1+m_Matrix./Km(ii)+k_Matrix./Kk(ii)).*x(ii)'./(Kh(ii).*P(ii).^2) );
pMBpH = -sum( (m_Matrix*x(ii)'./Km(ii))./(Kh(ii).*P(ii).^2) );
pMBpK = -sum( (m_Matrix*x(ii)'./Km(ii))./(Kk(ii).*P(ii).^2) );
pMBpM = +sum( (1+h_Matrix./Kh(ii)+k_Matrix./Kk(ii)).*x(ii)'./(Km(ii).*P(ii).^2) );
pKBpH = -sum( (k_Matrix*x(ii)'./Kk(ii))./(Kh(ii).*P(ii).^2) );
pKBpM = -sum( (k_Matrix*x(ii)'./Kk(ii))./(Km(ii).*P(ii).^2) );
pKBpK = +sum( (1+h_Matrix./Kh(ii)+m_Matrix./Km(ii)).*x(ii)'./(Kk(ii).*P(ii).^2) );
% PHIs
J_H = (0 + 0*J_ACON_Matrix) / VWater_Matrix;
J_M = (0) / VWater_Matrix;
J_K = (0) / VWater_Matrix;
Phi_H = J_H - sum( h_Matrix*f(ii)'./(Kh(ii).*P(ii)) );
Phi_M = -sum( m_Matrix*f(ii)'./(Km(ii).*P(ii)) );
Phi_K = J_K -sum( k_Matrix*f(ii)'./(Kk(ii).*P(ii)) );
% ALPHAs
aH = 1 + pHBpH;
aM = 1 + pMBpM;
aK = 1 + pKBpK;
% ADDITIONAL BUFFER for [H+]
aH = 1 + pHBpH + BX(1)/K_BX(1)/(1+h_Matrix/K_BX(1))^2; % M
% DENOMINATOR
D = aH*pKBpM*pMBpK + aK*pHBpM*pMBpH + aM*pHBpK*pKBpH - ...
    aM*aK*aH - pHBpK*pKBpM*pMBpH - pHBpM*pMBpK*pKBpH;
% DERIVATIVES for H,Mg,K
f(3,:) =  ( (pKBpM*pMBpK - aM*aK)*Phi_H + ...
            (aK*pHBpM - pHBpK*pKBpM)*Phi_M + ...
            (aM*pHBpK - pHBpM*pMBpK)*Phi_K ) / D;
f(4,:) =  ( (aK*pMBpH - pKBpH*pMBpK)*Phi_H + ...
            (pKBpH*pHBpK - aH*aK)*Phi_M + ...
            (aH*pMBpK - pHBpK*pMBpH)*Phi_K ) / D;
f(5,:) =  ( (aM*pKBpH - pKBpM*pMBpH)*Phi_H + ...
            (aH*pKBpM - pKBpH*pHBpM)*Phi_M + ...
            (pMBpH*pHBpM - aH*aM)*Phi_K ) / D;

%% ELECTROPHYS EQUATIONS
%% FLUX VECTOR:
J = [ J_ACON_Matrix];
